@@ -1,11 +1,11 @@
 import typing
 
 from ..concurrency import keepasync
-from .responses import Response, automatic
+from .responses import Response, convert
 from .request import Request
 
 
-MiddlewareMeta = keepasync("process_request", "process_response")
+MiddlewareMeta = keepasync("process_request", "process_response", "process_exception")
 
 
 class MiddlewareMixin(metaclass=MiddlewareMeta):  # type: ignore
@@ -24,19 +24,31 @@ class MiddlewareMixin(metaclass=MiddlewareMeta):  # type: ignore
         response = await self.process_request(request)
 
         if response is None:
-            response = await self.get_response(request)
+            try:
+                response = await self.get_response(request)
+            except Exception as exc:
+                response = await self.process_exception(request, exc)
+                if response is None:
+                    raise exc
 
-        if isinstance(response, tuple):
-            response = automatic(*response)
-        else:
-            response = automatic(response)
+        response = convert(response)
 
         response = await self.process_response(request, response)
 
         return response
 
-    async def process_request(self, request: Request) -> typing.Union[None, typing.Any]:
-        """Must return None, otherwise return the value as the result of this request."""
+    async def process_request(self, request: Request) -> typing.Optional[typing.Any]:
+        """
+        Must return None, otherwise return the value as the result of this request.
+        """
 
     async def process_response(self, request: Request, response: Response) -> Response:
         return response
+
+    async def process_exception(
+        self, request: Request, exception: Exception
+    ) -> typing.Optional[typing.Any]:
+        """
+        If return None, will raise exception.
+        Otherwise return the value as the result of this request.
+        """
