@@ -9,8 +9,9 @@ from decimal import Decimal
 from dataclasses import dataclass, field, InitVar
 
 from .types import Literal
-from .http.view import bound_params, HTTPView, only_allow
+from .utils import superclass
 from .concurrency import complicating
+from .http.view import bound_params, only_allow
 
 
 __all__ = [
@@ -77,7 +78,7 @@ class DecimalConvertor(Convertor):
     def convert(self, value: str) -> Decimal:
         return Decimal(value)
 
-    def to_string(self, value: Decimal) -> str:
+    def to_string(self, value: typing.Any) -> str:
         value = Decimal(value)
         if value < Decimal("0.0"):
             raise ValueError("Negative decimal are not supported")
@@ -319,6 +320,7 @@ class BaseRoute:
     def __post_init__(self) -> None:
         if not self.path.startswith("/"):
             raise ValueError("Route path must start with '/'")
+        self.endpoint = copy.copy(self.endpoint)
 
 
 @dataclass
@@ -367,33 +369,27 @@ class SocketRoute(BaseRoute):
 T = typing.TypeVar("T")
 
 
-class Routes(typing.List[BaseRoute]):
-    def __init__(
-        self,
-        *iterable: typing.Union["BaseRoute", "SubRoutes"],
-        http_middlewares: typing.List[typing.Any] = [],
-        socket_middlewares: typing.List[typing.Any] = [],
-    ) -> None:
-        routes = []
-        self.http_middlewares = copy.copy(http_middlewares)
-        self.socket_middlewares = copy.copy(socket_middlewares)
-        for route in iterable:
-            if not isinstance(route, Routes):
-                routes.append(route)
-                continue
-            for subroute in route:  # type: BaseRoute
-                subroute.extend_middlewares(route)
-                if isinstance(route, SubRoutes):
-                    routes.append(
-                        subroute.__class__(
-                            path=route.prefix + subroute.path,
-                            endpoint=subroute.endpoint,
-                            name=subroute.name,
-                        )
+class RouteRegisterMixin:
+    __slots__ = ()
+
+    def append(self, route: BaseRoute) -> None:
+        raise NotImplementedError()
+
+    def extend(self, routes: typing.List[BaseRoute]) -> None:
+        for route in routes:  # type: BaseRoute
+            if isinstance(routes, Routes):
+                route.extend_middlewares(routes)
+
+            if isinstance(routes, SubRoutes):
+                self.append(
+                    route.__class__(
+                        path=routes.prefix + route.path,
+                        endpoint=route.endpoint,
+                        name=route.name,
                     )
-                else:
-                    routes.append(subroute)
-        super().__init__(routes)
+                )
+            else:
+                self.append(route)
 
     def http(
         self,
@@ -415,14 +411,13 @@ class Routes(typing.List[BaseRoute]):
         if path and endpoint is None:
             # example: @router.http("/path", name="hello")
             #          async def func(request): ...
-            return lambda endpoint: self.http(path=path, endpoint=endpoint, name=name)
+            return lambda endpoint: self.http(
+                path=path, endpoint=endpoint, name=name, method=method
+            )
 
         if endpoint is None:
             raise ValueError("endpoint must be is not None")
 
-        if not issubclass(endpoint, HTTPView):
-            raise ValueError("endpoint must be inherit `HTTPView`")
-
         if name == "":
             name = endpoint.__name__
 
@@ -459,6 +454,34 @@ class Routes(typing.List[BaseRoute]):
 
         return endpoint
 
+
+class Routes(typing.List[BaseRoute], RouteRegisterMixin):
+    def __init__(
+        self,
+        *iterable: typing.Union["BaseRoute", "Routes"],
+        http_middlewares: typing.List[typing.Any] = [],
+        socket_middlewares: typing.List[typing.Any] = [],
+    ) -> None:
+        super().__init__()
+        self.http_middlewares = copy.copy(http_middlewares)
+        self.socket_middlewares = copy.copy(socket_middlewares)
+        for route in iterable:
+            if not isinstance(route, Routes):
+                self.append(route)
+            else:
+                self.extend(route)
+
+    def extend(self, routes: typing.List[BaseRoute]) -> None:  # type: ignore
+        """
+        Extend routes in routes
+
+        example:
+            routes.extend(Routes(...))
+        or
+            routes.extend([...])
+        """
+        return superclass(RouteRegisterMixin, self).extend(routes)
+
     def http_middleware(self, middleware: T) -> T:
         """
         append middleware in routes
@@ -523,7 +546,7 @@ class SubRoutes(Routes):
         )
 
 
-class Router:
+class Router(RouteRegisterMixin):
     def __init__(self, routes: typing.List[BaseRoute] = list()) -> None:
         self.http_tree = RadixTree()
         self.websocket_tree = RadixTree()
@@ -580,7 +603,7 @@ class Router:
             node.endpoint,
         )
 
-    def extend(self, routes: typing.List[BaseRoute] = list()) -> None:
+    def extend(self, routes: typing.List[BaseRoute]) -> None:
         """
         Add routes in router
 
@@ -589,10 +612,7 @@ class Router:
         or
             router.extend([...])
         """
-
-        for route in routes:
-            route.extend_middlewares(routes)
-            self.append(route)
+        return superclass(RouteRegisterMixin, self).extend(routes)
 
     def url_for(
         self,
@@ -618,66 +638,3 @@ class Router:
                 for name, value in path_params.items()
             }
         )
-
-    def http(
-        self,
-        path: str,
-        endpoint: typing.Any = None,
-        *,
-        name: str = "",
-        method: str = "",
-    ) -> typing.Any:
-        """
-        shortcut for `self.append`
-
-        example:
-            @router.http("/path", name="endpoint-name")
-            async def endpoint(request): ...
-        or
-            router.http("/path", endpoint, name="endpoint-name")
-        """
-        if path and endpoint is None:
-            # example: @router.http("/path", name="hello")
-            #          async def func(request): ...
-            return lambda endpoint: self.http(
-                path=path, endpoint=endpoint, name=name, method=method
-            )
-
-        if endpoint is None:
-            raise ValueError("endpoint must be is not None")
-
-        if name == "":
-            name = endpoint.__name__
-
-        self.append(HttpRoute(path, endpoint, name, method))
-
-        return endpoint
-
-    def websocket(
-        self, path: str, endpoint: typing.Any = None, *, name: str = ""
-    ) -> typing.Any:
-        """
-        shortcut for `self.append`
-
-        example:
-            @router.websocket("/path", name="endpoint-name")
-            async def endpoint(websocket): ...
-        or
-            router.websocket("/path", endpoint, name="endpoint-name")
-        """
-        if path and endpoint is None:
-            # example: @router.websocket("/path", name="hello")
-            #          async def func(websocket): ...
-            return lambda endpoint: self.websocket(
-                path=path, endpoint=endpoint, name=name
-            )
-
-        if endpoint is None:
-            raise ValueError("endpoint must be is not None")
-
-        if name == "":
-            name = endpoint.__name__
-
-        self.append(SocketRoute(path, endpoint, name))
-
-        return endpoint
